import os.path
import secrets

import boto3

from artifact_hub import s3_hub
from artifact_hub.backends.s3_backend import S3Backend
from artifact_hub.history_backends.history_s3_backend import HistoryS3Backend


def test_hub_initialization_s3_hub():
    folder = "folder"
    object_id = f"object_{secrets.token_hex(3)}"
    bucket_name = "iamr0b0tx-development"

    # create res dir if not existing. res is used to store resources(files) used by backends
    # if not os.path.exists("res"):
    #     os.mkdir("res")

    hub = s3_hub(bucket_name, folder, object_id)
    assert isinstance(hub, S3Backend)

    # download history file
    boto3.resource('s3').Bucket(bucket_name).download_file(f"{folder}/{object_id}/{HistoryS3Backend.HISTORY}",
                                                           HistoryS3Backend.HISTORY_PATH)
    with open(HistoryS3Backend.HISTORY_PATH) as file:
        assert file.read().startswith("0 ")

    random_idx = secrets.token_hex(3)
    sample_file_path = "res/sample_file.txt"
    lines = [f"{random_idx}: this is a sample file\n", "its has three lines\n", "This is the third line\n"]
    with open(sample_file_path, 'w') as file:
        file.writelines(lines)

    latest_version_id = hub.push(sample_file_path, "pushing sample file", "this is a test for sample file")
    assert latest_version_id is not None

    # create temp dir if not existing
    if not os.path.exists("temp"):
        os.mkdir("temp")

    hub.pull_into("temp", latest_version_id)
    with open(f"temp/{object_id}/{os.path.basename(sample_file_path)}") as file:
        assert file.readlines() == lines



import os

from artifact_hub.backends.s3_backend import S3Backend
from artifact_hub.history_backends.history_s3_backend import HistoryS3Backend

if not os.path.exists("res"):
    os.mkdir("res")


def s3_hub(bucket, folder, object_id):
    history = HistoryS3Backend(bucket, folder, object_id)
    backend = S3Backend(bucket, folder, object_id, history)
    return backend



import json
import urllib.request

import boto3
from botocore.exceptions import ClientError

from artifact_hub.history_backends.history_base_backend import HistoryBaseBackend


class HistoryS3Backend(HistoryBaseBackend):
    HISTORY = "history"
    HISTORY_PATH = "res/history"

    def __init__(self, bucket: str, folder: str, object_id):
        self.__repo_path = f"{folder}/{object_id}"
        self.__store = boto3.resource('s3').Bucket(bucket)

    @property
    def history(self):
        try:
            self.__store.download_file(f"{self.__repo_path}/{self.HISTORY}", self.HISTORY_PATH)
            with open(self.HISTORY_PATH) as file:
                history = list(map(lambda x: x.split(), file))

            return list(map(
                lambda y: (int(y[0]), y[1]),
                history
            ))

        except ClientError:
            return []

    def __sync(self, history):
        with open(self.HISTORY_PATH, 'w') as file:
            file.writelines(map(lambda x: f"{x[0]} {x[1]}\n", history))
        self.__store.upload_file(self.HISTORY_PATH, f"{self.__repo_path}/{self.HISTORY}")

    def add_new_version(self):
        history = self.history
        next_version_id = history[-1][0] + 1 if history else 0
        history.append((next_version_id, self.__get_current_datetime_string()))
        self.__sync(history.copy())
        return next_version_id

    @staticmethod
    def __get_current_datetime_string():
        return json.loads(
            urllib.request.urlopen(
                "https://www.timeapi.io/api/Time/current/zone?timeZone=UTC"
            ).read().decode()
        )["dateTime"]

    def get_latest_version_id(self) -> int:
        return self.history[-1][0]

    def is_empty(self) -> bool:
        return len(self.history) == 0



from abc import ABC, abstractmethod


class BaseBackend(ABC):
    @abstractmethod
    def pull_into(self, destination_path: str, version_id: int):
        pass

    @abstractmethod
    def push(self, source_path: str, title: str, description: str):
        pass



#!/usr/bin/env python
import sys
from io import open

from setuptools import find_packages, setup

CURRENT_PYTHON = sys.version_info[:2]
REQUIRED_PYTHON = (3, 6)

UNSUPPORTED_VERSION_INFO = f'''
==========================
Unsupported Python version
==========================
This version of Django Microservices Admin requires Python {REQUIRED_PYTHON}, but you're trying
to install it on Python {CURRENT_PYTHON}.
This may be because you are using a version of pip that doesn't
understand the python_requires classifier. Make sure you
have pip >= 9.0 and setuptools >= 24.2, then try again:
    $ python -m pip install --upgrade pip setuptools
    $ python -m pip install django-microservices-admin
This will install the latest version of Django Microservices Admin which works on
your version of Python.
'''

# This check and everything above must remain compatible with Python 2.7.
if CURRENT_PYTHON < REQUIRED_PYTHON:
    sys.stderr.write(UNSUPPORTED_VERSION_INFO)
    sys.exit(1)

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='artifact-hub',
    url='https://https://github.com/robotstech/artifact-hub',
    packages=find_packages(exclude=['tests*']),
    include_package_data=True,
)



import logging
import os

import boto3

from artifact_hub.backends.base_backend import BaseBackend
from artifact_hub.history_backends.history_base_backend import HistoryBaseBackend

logger = logging.getLogger("artifact_hub")


class S3Backend(BaseBackend):
    TITLE = "title"
    TITLE_PATH = "res/title"
    DESCRIPTION = "description"
    DESCRIPTION_PATH = "res/description"
    INIT = "init"
    INIT_PATH = "res/init"

    def __init__(self, bucket: str, folder: str, repo_id: str, history_backend: HistoryBaseBackend):
        self.__store = boto3.resource('s3').Bucket(bucket)
        self.__folder = folder
        self.__repo_id = repo_id
        self.__repo_base_path = f"{self.__folder}/{self.__repo_id}"
        self.__history = history_backend

        # check if repo init existed
        if self.__history.is_empty():
            logger.info(f"{repo_id} not found, creating now ...")
            self.__create_text_file(self.INIT_PATH, self.INIT)

            # create one if init does not exist
            self.push(
                self.INIT_PATH, f"{self.__repo_id}'s {self.INIT}",
                f"initializing {self.__repo_id} in the hub"
            )

            # feedback that repo is loaded
        logger.info(f"{repo_id} loaded")

    def __get_repo_version_path(self, version_id):
        return f"{self.__repo_base_path}/{version_id}"

    def __get_repo_path(self, version_id):
        return f"{self.__get_repo_version_path(version_id)}/{self.__repo_id}"

    def __get_repo_object_path(self, version_id, *args):
        return os.path.normpath(os.path.join(self.__get_repo_path(version_id), os.path.join(*args)))

    def __get_repo_version_description_path(self, version_id):
        return f"{self.__get_repo_version_path(version_id)}/{self.DESCRIPTION}"

    def __get_repo_version_title_path(self, version_id):
        return f"{self.__get_repo_version_path(version_id)}/{self.TITLE}"

    @staticmethod
    def __create_text_file(filepath: str, content: str):
        with open(filepath, 'w') as file:
            file.write(content)
        return filepath

    def pull_into(self, destination_path: str, version_id: int = None):
        if not version_id:
            version_id = self.__history.get_latest_version_id()

        repo_version_path = self.__get_repo_version_path(version_id)
        for obj in self.__store.objects.filter(Prefix=self.__get_repo_path(version_id)):
            obj_key_dest_path = os.path.normpath(os.path.join(destination_path, obj.key)) \
                .replace(repo_version_path, "", 1)

            if not os.path.exists(os.path.dirname(obj_key_dest_path)):
                os.makedirs(os.path.dirname(obj_key_dest_path))
            self.__store.download_file(obj.key, obj_key_dest_path)

    def push(self, source_path: str, title: str, description: str):
        try:
            version_id = self.__history.add_new_version()

            self.__store.upload_file(
                self.__create_text_file(self.TITLE_PATH, title),
                self.__get_repo_version_title_path(version_id)
            )

            self.__store.upload_file(
                self.__create_text_file(self.DESCRIPTION_PATH, description),
                self.__get_repo_version_description_path(version_id)
            )

            if os.path.isdir(source_path):
                for path, sub_dirs, files in os.walk(source_path):
                    for file in files:
                        dest_path = path.replace(source_path, "")
                        __s3file = self.__get_repo_object_path(version_id, dest_path, file)
                        __local_file = os.path.join(path, file)
                        self.__store.upload_file(__local_file, __s3file)
            else:
                self.__store.upload_file(
                    source_path,
                    self.__get_repo_object_path(version_id, os.path.basename(source_path))
                )

            return self.__history.get_latest_version_id()

        except Exception as e:
            logger.warning(f"could not push: {e}")



from abc import ABC, abstractmethod


class HistoryBaseBackend(ABC):
    @abstractmethod
    def get_latest_version_id(self) -> int:
        pass

    @abstractmethod
    def add_new_version(self) -> int:
        pass

    @abstractmethod
    def is_empty(self) -> bool:
        pass
